/**************************************************************************
 * This file is part of ESTmapper.
 * Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
 * Authors: Brian Walenz and Liliana Florea
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received (LICENSE.txt) a copy of the GNU General Public 
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/


#ifndef MT19937AR_H
#define MT19937AR_H

//  Refactoring of
//
//    A C-program for MT19937, with initialization improved 2002/1/26.
//    Coded by Takuji Nishimura and Makoto Matsumoto.
//
//  to make it thread safe and (hopefully) more portable.
//
//  20040421, bpw

//  bri.h contains the function prototypes, but we hide the structure and
//  implementation here.
//
#include "bri.h"

/* Period parameters */  
#define MT_N 624
#define MT_M 397
#define MT_MATRIX_A   0x9908b0dfUL /* constant vector a */
#define MT_UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define MT_LOWER_MASK 0x7fffffffUL /* least significant r bits */


struct mt {
  //  The array for the state vector
  //
  u32bit  mt[MT_N];

  //  The ordinal of the first uninitialized element --
  //  mti = N+1 -> element N is uninitialized
  //
  u32bit  mti;

  //  Something
  //  mag01[x] = x * MT_MATRIX_A  for x=0,1
  //
  u32bit  mag01[2];
};

//  This is declared in bri.h
//
//typedef struct mt mt_s;


#endif  //  MT19937AR_H
