// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "meryl.H"
#include "libmeryl.H"


void
unaryOperations(merylArgs *args) {

  if (args->mergeFilesLen != 1) {
    fprintf(stderr, "ERROR - must have exactly one file!\n");
    exit(1);
  }
  if (args->outputFile == 0L) {
    fprintf(stderr, "ERROR - no output file specified.\n");
    exit(1);
  }
  if ((args->personality != PERSONALITY_LEQ) &&
      (args->personality != PERSONALITY_GEQ) &&
      (args->personality != PERSONALITY_EQ)) {
    fprintf(stderr, "ERROR - only personalities lessthan, lessthanorequal,\n");
    fprintf(stderr, "ERROR - greaterthan, greaterthanorequal, and equal\n");
    fprintf(stderr, "ERROR - are supported in unaryOperations().\n");
    fprintf(stderr, "ERROR - this is a coding error, not a user error.\n");
    exit(1);
  }

  //  Open the input and output files -- we don't know the number
  //  unique, distinct, and total until after the operation, so we
  //  leave them zero.
  //
  merylStreamReader   *R = new merylStreamReader(args->mergeFiles[0]);
  merylStreamWriter   *W = new merylStreamWriter(args->outputFile, R->merSize(), R->prefixSize());

  switch (args->personality) {
    case PERSONALITY_LEQ:
      while (R->nextMer())
        if (R->theCount() <= args->desiredCount)
          W->addMer(R->theFMer(), R->theCount());
      break;

    case PERSONALITY_GEQ:
      while (R->nextMer())
        if (R->theCount() >= args->desiredCount)
          W->addMer(R->theFMer(), R->theCount());
      break;

    case PERSONALITY_EQ:
      while (R->nextMer())
        if (R->theCount() == args->desiredCount)
          W->addMer(R->theFMer(), R->theCount());
      break;
  }

  delete R;
  delete W;
}
