#!/usr/bin/perl

# This file is part of ESTmapper.
# Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
# Authors: Brian Walenz and Liliana Florea
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received (LICENSE.txt) a copy of the GNU General Public 
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



use FindBin;
use lib "$FindBin::Bin";
use libBri;
use strict;

if (scalar @ARGV == 0) {
    print STDERR "usage: $0 [list-of-id's or file-of-id's] < polishes > found\n";
    exit;
}

my $pattern;

#  Read the IDs from files or the command line, building up a
#  large regex to match them.
#
while (scalar(@ARGV) > 0) {
    my $arg = shift @ARGV;

    if (-e $arg) {
        open(F, "< $arg");
        while (<F>) {
            chomp;
            if (defined($pattern)) {
                $pattern .= "|$_";
            } else {
                $pattern = $_;
            }
        }
        close(F);
    } else {
        if (defined($pattern)) {
            $pattern .= "|$arg";
        } else {
            $pattern = $arg;
        }
    }
}

print STDERR "pattern = '$pattern'\n";

#  Now, read matches, outputting one if the edef matches the pattern

while (!eof(STDIN)) {
    my %p = libBri::readPolish(*STDIN);

    #print "$p{'estDefLine'}\n";

    if (defined($p{'raw'}) && ($p{'estDefLine'} =~ m/$pattern/i)) {
        print STDERR "$p{'estDefLine'}\n";
        print STDOUT $p{'raw'};
    }
}
