#!/usr/bin/perl

# This file is part of ESTmapper.
# Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
# Authors: Brian Walenz and Liliana Florea
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received (LICENSE.txt) a copy of the GNU General Public 
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



$| = 1;
use strict;
use lib "/work/assembly/walenzbp/projects/scripts";
use libBri;


#  Takes a fasta file and a set of polishes, and rewrites them both,
#  removing sequences that aren't used.
#
#  For example, if the polishes only use cDNA #392 out of 4000, then
#  the resulting fasta will contain just cDNA #392, and the index in
#  the output polishes file will be correct.
#

if (scalar(@ARGV) != 4) {
    print STDERR "usage: $0 in.polishes in.fasta out.polishes out.fasta\n";
    exit;
}

die "Can't find $ARGV[0]\n" if (! -e $ARGV[0]);
die "Can't find $ARGV[1]\n" if (! -e $ARGV[1]);
#die "File exists: $ARGV[2]\n" if (-e $ARGV[2]);
#die "File exists: $ARGV[3]\n" if (-e $ARGV[3]);

#
#  Read the polishes
#

my %IDXs;
my $idx = 0;

open(F, "< $ARGV[0]");
open(O, "> $ARGV[2]");
while (!eof(F)) {
    my %p = &libBri::readPolish(*F);
    if (defined($p{'raw'})) {
        if (!defined($IDXs{$p{'estID'}})) {
            $IDXs{$p{'estID'}} = $idx;
            $idx++;
        }

        $p{'estID'} = $IDXs{$p{'estID'}};
        print O &libBri::updatePolishInfoLine(%p);
    }
}
close(F);
close(O);

print STDERR "Calling leaff for " . scalar(keys %IDXs) . " fasta sequences.\n";


#
#  Use leaff to rewrite the fasta
#
open(O, "> junk.$$.reducePolishes");
foreach my $k (sort {$a <=> $b} keys %IDXs) {
    print O "$k\n";
}
close(O);

system("/work/assembly/walenzbp/releases/leaff -F $ARGV[1] -q junk.$$.reducePolishes > $ARGV[3]");

unlink "junk.$$.reducePolishes";
