// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "posix.H"
#include <stdio.h>
#include <stdlib.h>
#include "seatac.H"
#include "bri++.H"

encodedQuery::encodedQuery(char const    *seq,
                           u32bit         seqLen,
                           u32bit         k,
                           bool           rc) {
  _seq            = seq;
  _seqLen         = seqLen;
  _merSize        = k;
  _rc             = rc;

  _seqPos         = 0;

  _substring      = u64bitZERO;
  _mermask        = u64bitMASK(2 * _merSize);
  _timeUntilValid = _merSize;
}


bool
encodedQuery::getMer(u64bit &mer, u32bit &pos) {
  bool  found = false;

  mer = u64bitZERO;
  pos = u32bitZERO;

  if (_rc) {

    while (!found && (_seqPos < _seqLen)) {
      _substring <<= 2;
      _substring  &= _mermask;

      if (validSymbol[_seq[_seqLen - 1 - _seqPos]]) {
        _substring |= compressSymbol[ complementSymbol[ _seq[_seqLen - 1 - _seqPos] ]];
        _timeUntilValid--;
      } else {
        _timeUntilValid = _merSize;
      }

      _seqPos++;

      if (_seqPos >= _merSize) {
        mer   = _substring;
        pos   = _seqPos - _merSize;
        found = _timeUntilValid <= 0;
      }
    }

  } else {

    while (!found && (_seqPos < _seqLen)) {
      _substring <<= 2;
      _substring  &= _mermask;

      if (validSymbol[_seq[_seqPos]]) {
        _substring |= compressSymbol[ _seq[_seqPos] ];
        _timeUntilValid--;
      } else {
        _timeUntilValid = _merSize;
      }

      _seqPos++;

      if (_seqPos >= _merSize) {
        mer   = _substring;
        pos   = _seqPos - _merSize;
        found = _timeUntilValid <= 0;
      }
    }

  }

  return(found);
}




encodedQuery::~encodedQuery() {
}
