// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "posix.H"
#include "hitMatrix.H"

//  $Id: hitMatrix-sort.C,v 1.1 2003/10/21 03:55:56 walenz Exp $

//
//  Sort by dsPos
//

inline
void
adjustHeap_dsPos(diagonalLine *L, s32bit p, s32bit n) {
  u32bit  q = L[p]._qsPos;
  u32bit  d = L[p]._dsPos;
  u32bit  l = L[p]._diagonalID;
  s32bit  c = (p << 1) + 1;  //  let c be the left child of p

  while (c < n) {

    //  Find the larger of the two children
    //
    if ((c+1 < n) && (L[c]._dsPos < L[c+1]._dsPos))
      c++;

    //  Does the node in question fit here?
    //
    if (d >= L[c]._dsPos)
      break;

    //  Else, swap the parent and the child
    //
    L[p]._qsPos      = L[c]._qsPos;
    L[p]._dsPos      = L[c]._dsPos;
    L[p]._diagonalID = L[c]._diagonalID;

    //  Move down the tree
    //
    p = c;
    c = (p << 1) + 1;
  }

  L[p]._qsPos      = q;
  L[p]._dsPos      = d;
  L[p]._diagonalID = l;
}

void
hitMatrix::sort_dsPos(void) {

  if (_hitsLen > 1) {

    //  Create the heap of lines.
    //
    for (s32bit i=_hitsLen/2-1; i>=0; i--)
      adjustHeap_dsPos(_hits, i, _hitsLen);

    //  Interchange the new maximum with the element at the end of the tree
    //
    for (u32bit i=_hitsLen-1; i>0; i--) {
      u32bit  q  = _hits[i]._qsPos;
      u32bit  d  = _hits[i]._dsPos;
      u32bit  l  = _hits[i]._diagonalID;

      _hits[i]._qsPos      = _hits[0]._qsPos;
      _hits[i]._dsPos      = _hits[0]._dsPos;
      _hits[i]._diagonalID = _hits[0]._diagonalID;

      _hits[0]._qsPos      = q;
      _hits[0]._dsPos      = d;
      _hits[0]._diagonalID = l;
      
      adjustHeap_dsPos(_hits, 0, i);
    }
  }
}
