// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef SHAREDOBJ_H
#define SHAREDOBJ_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <errno.h>

//  A wrapper around a shared object.
//
//  Responsible for opening, accessing and closing a shared object.
//

class sharedObj {
public:

  sharedObj(char *p) {
    path = new char [strlen(p) + 1];
    strcpy(path, p);

    handle = dlopen(path, RTLD_NOW);
    if (handle == 0L) {
      fprintf(stderr, "ERROR: Failed to open shared object '%s'\n%s\n", path, dlerror());
      exit(1);
    }
  };

  ~sharedObj() {
    if (dlclose(handle)) {
      fprintf(stderr, "WARNING: Failed to close shared object '%s'\n%s\n", path, dlerror());
    }
    delete [] path;
  };

  bool    exists(char *name) {
    void *ptr = 0L;

    errno = 0;
    ptr = dlsym(handle, name);
    if (errno) {
      fprintf(stderr, "ERROR: Failed to find symbol '%s' in shared object '%s'\n%s\n", name, path, dlerror());
      exit(1);
    }

    return(ptr != 0L);
  }

  void   *get(char *name) {
    void *ptr = 0L;

    errno = 0;
    ptr = dlsym(handle, name);
    if (errno) {
      fprintf(stderr, "ERROR: Failed to find symbol '%s' in shared object '%s'\n%s\n", name, path, dlerror());
      exit(1);
    }
    if (ptr == 0L) {
      fprintf(stderr, "ERROR: Symbol '%s' not present in shared object '%s'\n", name, path);
      exit(1);
    }

    return(ptr);
  };

private:
  char *path;
  void *handle;
};


#endif  //  SHAREDOBJ_H
