// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <math.h>
#include "sim4.H"


int
Sim4::align_get_dist(int i1, int j1, int i2, int j2, int limit)
{
  int *last_d, *temp_d;
  int goal_diag, ll, uu;
  int c, k, row;
  int start, lower, upper;
  
  /* Compute the boundary diagonals */
  start = j1 - i1;
  lower = max(j1-i2, start-limit);
  upper = min(j2-i1, start+limit);
  goal_diag = j2-i2;
  
  if (goal_diag > upper || goal_diag < lower)
    return(-1);
  
  /* Allocate space for forward vectors */ 
  last_d = (int *)ckalloc((upper-lower+1)*sizeof(int)) - lower;
  temp_d = (int *)ckalloc((upper-lower+1)*sizeof(int)) - lower;

  /* Initialization */
  for (k=lower; k<=upper; ++k)
    last_d[k] = -2147483647;  //  INT_MIN;

  last_d[start] = snake(start, i1, i2, j2);
  
  if (last_d[goal_diag] >= i2) {
    /* Free working vectors */
    ckfree(last_d+lower);
    ckfree(temp_d+lower);
    return 0;
  }

  for (c=1; c<=limit; ++c) {
    ll = max(lower,start-c); uu = min(upper, start+c);
    for (k=ll; k<=uu; ++k) {
      if (k == ll)
        row = last_d[k+1]+1;    /* DELETE */
      else if (k == uu)
        row = last_d[k-1];      /* INSERT */
      else if ((last_d[k]>=last_d[k+1]) &&
               (last_d[k]+1>=last_d[k-1]))
        row = last_d[k]+1;      /*SUBSTITUTE */
      else if ((last_d[k+1]+1>=last_d[k-1]) &&
               (last_d[k+1]>=last_d[k]))
        row = last_d[k+1]+1;    /* DELETE */
      else
        row = last_d[k-1];      /* INSERT */
      
      temp_d[k] = snake(k,row,i2,j2);
    }     
    
    for (k=ll; k<=uu; ++k) last_d[k] = temp_d[k];

    if (last_d[goal_diag] >= i2) {
      /* Free working vectors */
      ckfree(last_d+lower);
      ckfree(temp_d+lower);
      return c;
    }
  }

  ckfree(last_d+lower);
  ckfree(temp_d+lower);

  /* Ran out of distance limit */
  return -1;
}

int
Sim4::get_dist(int i1, int j1, int i2, int j2, int limit)
{
        int *SS, *DD, *II;
        int goal_diag;
        int c, k, t1, t2, t;
        int start, lower, upper;

        /* Compute the boundary diagonals */
        start = j1 - i1;
        lower = max(j1-i2, start-limit);
        upper = min(j2-i1, start+limit);
        goal_diag = j2-i2;

        if (goal_diag > upper || goal_diag < lower) {
                printf("The two sequences are not really similar.(1 %d; %d %d %d %d)\n", limit, i1, j1, i2, j2);
                printf("Please try exact phase 1 method\n.");
                exit(1);
        }

        /* Allocate space for forward vectors */
        SS = (int *)ckalloc((upper-lower+1)*sizeof(int)) - lower;
        DD = (int *)ckalloc((upper-lower+2)*sizeof(int)) - lower;
        II = (int *)ckalloc((upper-lower+2)*sizeof(int)) - lower + 1;

        /* Initialization */
        for (k=lower; k<=upper; ++k) SS[k] = -99999;
        for (k=lower; k<=upper+1; ++k) DD[k] = -99999;
        for (k=lower-1; k<=upper; ++k) II[k] = -99999;
        SS[start] = snake(start, i1, i2, j2);

        if (SS[goal_diag] >= i2) {
#ifdef STATS
                printf("get_dist = %d\n", 0);
#endif

                /* Free working vectors */
                free(SS+lower);
                free(DD+lower);
                free(II+lower-1);
                return 0;
        }

        for (c=1; c<=limit; ++c) {
                t = max(lower, start-c);
                t1 = II[t-1];
                for (k=t; k<=min(upper, start+c); ++k) {
                        t2 = II[k];
                        II[k] = max(t1, SS[k]);
                        t1 = t2;
                        DD[k] = max(DD[k+1]+1, SS[k]);
                        SS[k] = snake(k, min(j2-k,max(max(SS[k]+1, II[k]), DD[k]
)), i2, j2);
                }

                if (SS[goal_diag] >= i2) {
#ifdef STATS
                        printf("get_dist = %d\n", c);
#endif

                        /* Free working vectors */
                        free(SS+lower);
                        free(DD+lower);
                        free(II+lower-1);
                        return c;
                }
        }

        /* Ran out of distance limit */
        printf("Two sequences are not really similar.\n");
        printf("Please try exact phase 1.\n");
        exit(1);
}



/* Condense_both_Ends  --  merge contiguous operations of the same type    */
/* together; return both new ends of the chain.                            */
void
Sim4::Condense_both_Ends(edit_script **head,
                         edit_script **tail,
                         edit_script **prev) {
  edit_script *tp, *tp1;

  tp = *head; *prev = NULL;
  while (tp != NULL) {
    while (((tp1 = tp->next) != NULL) && (tp->op_type == tp1->op_type)) {
      tp->num = tp->num + tp1->num;
      tp->next = tp1->next;
      ckfree(tp1);
    }
    if (tp->next) *prev = tp;
    else *tail = tp;
    tp = tp->next;
  }
}




void
Sim4::pluri_align(int *dist_ptr,
                  Exon *theExons,
                  struct edit_script_list **Aligns,
                  sim4_stats_t *st) {
  int    i, end1, end2, diff, ali_dist;
  char *a, *b;

  Exon  *thisExon = theExons;
  Exon  *nextExon;

  int    EditDistance = 0;     //  Sum of all tmpi, previously known as TMPI
  int    AlignmentLength = 0;

  struct edit_script_list *enew;
  struct edit_script      *head;
  struct edit_script      *tmp_script;
  struct edit_script      *left;
  struct edit_script      *right;
  struct edit_script      *prev;

  st->numberOfMatches = 0;
  st->numberOfNs      = 0;

  head      = 0L;
  *Aligns   = 0L;
  *dist_ptr = ali_dist = 0;

  end1 = _genLen; 
  end2 = _estLen;

  nextExon = thisExon->next_exon;

  while (nextExon && nextExon->toGEN) {
    diff = thisExon->frEST - nextExon->toEST - 1;
      
    if (diff != 0) {
      if (thisExon->toGEN) {
        enew = (edit_script_list *)ckalloc(sizeof(edit_script_list));
        enew->next_script = *Aligns;
        *Aligns = enew;
        (*Aligns)->script = head;
        (*Aligns)->offset1 = thisExon->frGEN;
        (*Aligns)->offset2 = thisExon->frEST;
        (*Aligns)->len1 = end1-(*Aligns)->offset1+1;
        (*Aligns)->len2 = end2-(*Aligns)->offset2+1;
        (*Aligns)->score = ali_dist;
        ali_dist = 0;
        head = NULL;
      }
      end1 = nextExon->toGEN;
      end2 = nextExon->toEST;  
    } else {
      diff = thisExon->frGEN - nextExon->toGEN - 1;
      if (diff != 0) {
        if (thisExon->toGEN) {
          struct edit_script  *newthing;

          newthing = (edit_script *) ckalloc(sizeof(edit_script));
          newthing->op_type = DELETE;
          newthing->num     = diff;
          newthing->next    = head;
          head = newthing;
        } else {
          end1 = nextExon->toGEN;
        }
      }
    }

    if (globalParams->_interspecies)
       diff = get_dist(nextExon->frGEN-1, nextExon->frEST-1,
                       nextExon->toGEN, nextExon->toEST,
                       max(1000, (int)(globalParams->_percentError*(nextExon->toEST - nextExon->frEST + 1))));
    else // original 
       diff = align_get_dist(nextExon->frGEN-1, nextExon->frEST-1,
                             nextExon->toGEN, nextExon->toEST,
                             max(1000, (int)(globalParams->_percentError*(nextExon->toEST - nextExon->frEST + 1))));


    //  Return if the alignment fails.
    //
    if (diff < 0) {
      st->numberOfMatches = 0;
      st->numberOfNs      = 0;
      st->percentID       = -1;

      *Aligns             = 0L;

      return;
    }

#ifdef STATS
    if (diff>P*(nextExon->toEST-nextExon->frEST+1))
      (void)printf("Warning: Distance threshold on segment exceeded.\n");
#endif

    if (globalParams->_interspecies)
       path(nextExon->frGEN-1, nextExon->frEST-1, SUBSTITUTE,
            nextExon->toGEN, nextExon->toEST, SUBSTITUTE,
            diff, &left, &right);
    else // original
       align_path(nextExon->frGEN-1, nextExon->frEST-1, 
                  nextExon->toGEN, nextExon->toEST, diff, &left, &right);


    Condense_both_Ends(&left, &right, &prev);
    
    if (!thisExon->toGEN && right->op_type == DELETE) {
      /* remove gaps at end of alignment */
      diff -= 0+right->num;         /* subtract GAP_OPEN = 0 */
      nextExon->toGEN -= right->num;
      end1 -= right->num; 
      if (head && (head->op_type == DELETE)) 
        head->num += right->num;
      ckfree(right);
      prev->next = NULL; 
      right = prev;
    } 

    if ((!nextExon->next_exon || !nextExon->next_exon->toGEN) &&
        left && (left->op_type == DELETE)) {
      diff -= 0+left->num;          /* subtract GAP_OPEN = 0 */
      nextExon->frGEN += left->num;

      tmp_script = left->next; 
      if (right == left)
        right = tmp_script;
      ckfree(left);
      left = tmp_script; 
    }
    
    *dist_ptr += diff;
    ali_dist += diff;

    a = _genSeq + nextExon->frGEN - 1;
    b = _estSeq + nextExon->frEST - 1;

    nextExon->numMatches = 0;
    nextExon->numNs      = 0;
    nextExon->numInDel   = 0;
    nextExon->numEdits   = 0;

    tmp_script = left;  

    //  These are used during SUBSTITUTE below to tell if the base at
    //  a (b) is N (upper or lower case).
    //
    bool  an = false;
    bool  bn = false;

    while (tmp_script) {
      switch (tmp_script->op_type) {
      case  DELETE:
        nextExon->numInDel += tmp_script->num;
        nextExon->numEdits += tmp_script->num;
        a                  += tmp_script->num;
        break;
      case  INSERT:
        nextExon->numInDel += tmp_script->num;
        nextExon->numEdits += tmp_script->num;
        b                  += tmp_script->num;
        break;
      case  SUBSTITUTE:

        //  Count the number of matches and edits.
        //
        //  An edit is a true substitute -- a base for a different base,
        //  not a base for an 'n'.
        //
        for (i=0; i<tmp_script->num; ++i, ++a, ++b) {

          an = (*a == 'N') || (*a == 'n');
          bn = (*b == 'N') || (*b == 'n');
            

          if (an && bn) {
            //  Both are N.  It isn't a match and it isn't an edit.
            //
            nextExon->numNs++;
          } else if (an || bn) {
            //  One is an N.  Someone has low quality sequence, and we
            //  should penalize.  We need to special case this because
            //  IUPACidentity[][] claims N matches all.
            //
            nextExon->numEdits++;
          } else if (IUPACidentity[(int)*a][(int)*b]) {
            //  Got a match.
            nextExon->numMatches++;
          } else {
            //  Got a substitution
            nextExon->numEdits++;
          }
        }
        break;
      }
      tmp_script = tmp_script->next;
    }

    nextExon->alignmentLength = (nextExon->toGEN - nextExon->frGEN + 1 +
                                 nextExon->toEST - nextExon->frEST + 1 +
                                 nextExon->numInDel);
    nextExon->percentID = (int)floor(100*(1 - 2 * nextExon->numEdits /
                                          (float)(nextExon->alignmentLength)));


    st->numberOfMatches += nextExon->numMatches;
    st->numberOfNs      += nextExon->numNs;

    EditDistance    += nextExon->numEdits;
    AlignmentLength += (nextExon->toGEN - nextExon->frGEN + 1 +
                        nextExon->toEST - nextExon->frEST + 1 +
                        nextExon->numInDel);

    right->next = head;
    head = left;

    thisExon = nextExon;
    nextExon = thisExon->next_exon;
  }


  /* at the beginning of the sequences */
  if (nextExon!=NULL) {

    if ((diff=thisExon->frEST-nextExon->toEST-1)!=0 && (diff != _estLen)) {
      enew = (edit_script_list *)ckalloc(sizeof(edit_script_list));
      enew->next_script = *Aligns;
      *Aligns = enew;
      (*Aligns)->offset1 = thisExon->frGEN;
      (*Aligns)->offset2 = thisExon->frEST;
      (*Aligns)->len1 = end1-(*Aligns)->offset1+1;
      (*Aligns)->len2 = end2-(*Aligns)->offset2+1;
      (*Aligns)->script = head;
      (*Aligns)->score = ali_dist;
      
    } else if (diff != _estLen) {

      /* modified to cut introns at the beginning of the sequence */
      enew = (edit_script_list *)ckalloc(sizeof(edit_script_list));
      enew->next_script = *Aligns;
      *Aligns = enew;
      (*Aligns)->offset1 = thisExon->frGEN;
      (*Aligns)->offset2 = 1;
      (*Aligns)->len1 = end1-(*Aligns)->offset1+1;
      (*Aligns)->len2 = end2-(*Aligns)->offset2+1;
      (*Aligns)->script = head;
      (*Aligns)->score = ali_dist;
    }
  }

  st->percentID = 0;
  if (AlignmentLength > 0)
    st->percentID = (int)floor(100 * (1 - 2 * (double)EditDistance / (double)AlignmentLength));
}




void
Sim4::updateStatistics(Exon *theExon,
                       sim4_stats_t *st) {

  theExon = theExon->next_exon;

  st->numberOfMatches = 0;
  st->numberOfNs      = 0;

  int    EditDistance    = 0;
  int    AlignmentLength = 0;

  while (theExon && theExon->toGEN) {
    st->numberOfMatches += theExon->numMatches;
    st->numberOfNs      += theExon->numNs;

    EditDistance    += theExon->numEdits;
    AlignmentLength += (theExon->toGEN - theExon->frGEN + 1 +
                        theExon->toEST - theExon->frEST + 1 +
                        theExon->numInDel);

    theExon = theExon->next_exon;
  }

  st->percentID = 0;
  if (AlignmentLength > 0)
    st->percentID = (int)floor(100 * (1 - 2 * (double)EditDistance / (double)AlignmentLength));
}
