// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "bri++.H"
#include "sim4polish.h"

//  Build an interval list with all exons (from both guys), merge
//  overlapping regions, compute the length, subtract from the total.
//  Result: the number of bp that the two matches overlap in the
//  genomic.
//
u32bit
findOverlap(sim4polish *A, sim4polish *B) {

  if ((A->genID != B->genID) || (A->matchOrientation != B->matchOrientation))
    return(0);

  u32bit        length = 0;
  u32bit        total  = 0;
  intervalList  IL;

  for (u32bit i=0; i<A->numExons; i++) {
    length = A->exons[i].genTo - A->exons[i].genFrom + 1;
    total  += length;
    IL.add(A->genLo + A->exons[i].genFrom, length);
  }

  for (u32bit i=0; i<B->numExons; i++) {
    length = B->exons[i].genTo - B->exons[i].genFrom + 1;
    total  += length;
    IL.add(B->genLo + B->exons[i].genFrom, length);
  }

  IL.merge();

  return(total - IL.sumOfLengths());
}
