/**************************************************************************
 * This file is part of ESTmapper.
 * Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
 * Authors: Brian Walenz and Liliana Florea
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received (LICENSE.txt) a copy of the GNU General Public 
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

//  Touches a chunk of memory, then exits.
//
//  If there is an argument on the command line, it is treated as the
//  number of megabytes to allocate.

struct memory {
  unsigned long  *data;
  struct memory  *next;
};


int
main(int argc, char **argv) {
  struct memory   *M  = NULL;
  int              s  = 256;
  int              ps = 0;
  int              i  = 0;
  int              j  = 0;
  int              k  = 0;
  int              r  = 0;

  if (argc == 2)
    s = (size_t)atoi(argv[1]);

  ps = getpagesize();

  fprintf(stderr, "Flushing %luMB (pagesize = %u bytes).\n", s, ps);

  errno = 0;
  M = (struct memory *)malloc(sizeof(struct memory) * s);
  if (errno) {
    fprintf(stderr, "Can't allocate %d struct memory's: %s\n", s, strerror(errno));
    exit(1);
  }

  for (i=0; i<s; i++) {
    errno = 0;
    M[i].data = (unsigned long *)malloc(1048576);
    if (errno) {
      fprintf(stderr, "Can't allocate region at %dMB: %s\n", i, strerror(errno));
      exit(1);
    }

#if 0
    for (k=0; k<=i; k++)
      for (j=0; j<1048576 / sizeof(unsigned long); j += ps)
        M[k].data[j] = 0;
#endif
  }

  fprintf(stderr, "Finished.  Kill me to exit.\n");

  ps = 1048576 / sizeof(unsigned long);

  while (1) {
    fprintf(stderr, "Loop!  r=%lu\n", r);
    for (k=0; k<s; k++) {
      j = 0;
      while (j < ps) {
        M[k].data[j] += M[k].data[ (j + r) % ps ] + 1;
        r += M[k].data[j];
        j += (r % 13) + 1;
      }
    }
  }

  return(0);
}
