/**************************************************************************
 * This file is part of ESTmapper.
 * Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
 * Authors: Brian Walenz and Liliana Florea
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received (LICENSE.txt) a copy of the GNU General Public 
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/time.h>


double
getTime(void) {
  struct timeval  tp;
  gettimeofday(&tp, NULL);
  return(tp.tv_sec + (double)tp.tv_usec / 1000000.0);
}


int
main(int argc, char **argv) {
  int      i;
  int      cmdL = 0;
  char    *cmd  = NULL;
  struct   utsname un;
  struct   rusage  ru;
  double   startTime = getTime();

  if (argc == 1) {
    fprintf(stderr, "usage: %s <cmd to run and all arguments>\n", argv[0]);
    exit(1);
  }

  for (i=1; i<argc; i++)
    cmdL += strlen(argv[i]) + 1;

  errno = 0;
  cmd = (char *)malloc(sizeof(char) * cmdL + 1);
  if (errno) {
    fprintf(stderr, "Can't allocate %d bytes to store the command!\n%s\n", cmdL, strerror(errno));
    exit(1);
  }

  cmd[0] = 0;

  for (i=1; i<argc; i++) {
    strcat(cmd, argv[i]);
    strcat(cmd, " ");
  }

  errno = 0;
  system(cmd);
  if (errno) {
    fprintf(stderr, "system() failed.\n%s\n", strerror(errno));
  }


  fprintf(stdout, "uname()---------------------------------\n");
  errno = 0;
  if (uname(&un) == -1) {
    fprintf(stdout, "uname() call failed: %s\n", strerror(errno));
  } else {
    fprintf(stdout, "sysname:     %s\n", un.sysname);
    fprintf(stdout, "nodename:    %s\n", un.nodename);
    fprintf(stdout, "release:     %s\n", un.release);
    fprintf(stdout, "version:     %s\n", un.version);
    fprintf(stdout, "machine:     %s\n", un.machine);
  }

  fprintf(stdout, "\n");

  fprintf(stdout, "rusage()--------------------------------\n");
  fprintf(stdout, "wallTime:    %f\n", getTime() - startTime);
  errno = 0;
  if (getrusage(RUSAGE_CHILDREN, &ru) == -1) {
    fprintf(stdout, "getrusage() call failed: %s\n", strerror(errno));
  } else {
    fprintf(stdout, "userTime:    %f\n", ru.ru_utime.tv_sec + (double)ru.ru_utime.tv_usec / 1000000.0);
    fprintf(stdout, "systemTime:  %f\n", ru.ru_stime.tv_sec + (double)ru.ru_stime.tv_usec / 1000000.0);
    fprintf(stdout, "maxrss:      %ld\n", ru.ru_maxrss);
    fprintf(stdout, "ixrss:       %ld\n", ru.ru_ixrss);
    fprintf(stdout, "idrss:       %ld\n", ru.ru_idrss);
    fprintf(stdout, "isrss:       %ld\n", ru.ru_isrss);
    fprintf(stdout, "minflt:      %ld\n", ru.ru_minflt);
    fprintf(stdout, "majflt:      %ld\n", ru.ru_majflt);
    fprintf(stdout, "nswap:       %ld\n", ru.ru_nswap);
    fprintf(stdout, "inblock:     %ld\n", ru.ru_inblock);
    fprintf(stdout, "oublock:     %ld\n", ru.ru_oublock);
    fprintf(stdout, "msgsnd:      %ld\n", ru.ru_msgsnd);
    fprintf(stdout, "msgrcv:      %ld\n", ru.ru_msgrcv);
    fprintf(stdout, "nsignals:    %ld\n", ru.ru_nsignals);
    fprintf(stdout, "nvcsw:       %ld\n", ru.ru_nvcsw);
    fprintf(stdout, "nivcsw:      %ld\n", ru.ru_nivcsw);
  }

  fprintf(stdout, "\n");
}
