// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include <stdlib.h>
#include "fasta.H"

//  Reads a searchGENOME hitCount file, filters the cDNA from that
//  search by the number of hits.
//
//  If the cutoff is > 0, sequences with count >= cutoff are output.
//  If the cutoff is < 0, sequences with count <= cutoff are output.
//  If the cutoff is = 0, sequences with count == zero are output.
//

#define MAX_ESTS    (16 * 1024 * 1024)

int
main(int argc, char **argv) {

  if (argc != 4) {
    fprintf(stderr, "usage: %s <cutoff> <est-hit-counts> <src.fasta>\n", argv[0]);
    exit(1);
  }

  int   *hitCounts = new int [MAX_ESTS];
  int    numESTs   = 0;
  int    cutoff    = atoi(argv[1]);
  int    i         = 0;

  FILE *F = fopen(argv[2], "r");
  while (!feof(F)) {
    fscanf(F, " %d ", hitCounts + numESTs);
    numESTs++;
  }
  fclose(F);

  FastABuffer  B;
  FastA        I(argv[3], true, false);

  for (I.first(B), i=0; I.eof() == false; i++, I.next(B)) {
    if ((cutoff < 0) && (hitCounts[i] <= -cutoff))
      fprintf(stdout, "%s\n%s\n", B.header(), B.sequence());
    if ((cutoff > 0) && (hitCounts[i] >= cutoff))
      fprintf(stdout, "%s\n%s\n", B.header(), B.sequence());
    if ((cutoff == 0) && (hitCounts[i] == 0))
      fprintf(stdout, "%s\n%s\n", B.header(), B.sequence());
  }

  if (i > numESTs)
    fprintf(stderr, "Found more sequences in the fasta than in the count?\n");
  if (i < numESTs)
    fprintf(stderr, "Found fewer sequences in the fasta than in the count?\n");

  return(0);
}

