// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include "fasta.H"

//  Does masking ala sim4 to an input fasta file.  This probably
//  doesn't do what you want it to, so be sure to understand the
//  output format (in the pile of #ifs near the end).
//

//  Directly from poly.C, but here because of link errors.
//
void
get_polyAT(unsigned char *seq, int len, int *pT, int *pA) {
  int i, sum10, sum20;
  unsigned char *s, *t, *v;
  int last10;

  int MAX10 = 2;
  int MAX20 = 5;

  char encodingA[128];
  char encodingT[128];

  if (pA) {
    for (int z=128; z; )
      encodingA[--z] = 1;

    encodingA['A'] = encodingA['X'] = encodingA['N'] = 0;

    for (i=0, s=seq+len, sum10=0, last10=len+1; i<10 && s>seq && sum10<=MAX20; i++)  {
      sum10 += encodingA[*(--s)];
    }

    t = v = seq+len;
    sum20 = sum10;
    for ( ; s>=seq && (sum10<=MAX10 || sum20<=MAX20); ) {
      if (!encodingA[*s] && sum10<=MAX10 && (seq+len>=s+20 || sum20<MAX10))
        last10 = s-seq+1;
      if (--s>seq) {
        sum10 += encodingA[*s] - encodingA[*(--t)];
        sum20 += encodingA[*s] -(((seq+len)-s>20) ? encodingA[*(--v)] : 0);
      }
    }

    if (last10>len-10) {
      *pA = len+1;
    } else {
      s = seq+last10+8;
      while (s >= seq && !encodingA[*s]) s--;
      if ((s-seq+1)-last10+1<=5)
        *pA = s-seq+2;
      else
        *pA = last10;
    }

    *pA = len-(*pA)+1;
  }


  if (pT) {
    for (int z=128; z; )
      encodingT[--z] = 1;

    encodingT['T'] = encodingT['X'] = encodingT['N'] = 0;

    for (i=0, s=seq-1, sum10=0, last10=0; i<10 && i<len-1 && sum10<=MAX20; i++) {
      sum10 += encodingT[*(++s)];
    }

    t = v = seq-1;
    sum20 = sum10;
    for ( ; s<seq+len && (sum10<=MAX10 || sum20<=MAX20); ) {
      if (!encodingT[*s] && sum10<=MAX10 && (s-seq>=19 || sum20<MAX10))
        last10 = s-seq+1;
      if (++s<seq+len) {
        sum10 += encodingT[*s] - encodingT[*(++t)];
        sum20 += encodingT[*s] - ((s-seq>=20) ? encodingT[*(++v)] : 0);
      }
    }

    if (last10<=10) *pT = 0;
    else {
      s = seq+last10-10;
      while (s < seq+len && !encodingT[*s]) s++;
      if (last10-(s-seq)+1<=5)
        *pT = s-seq;
      else
        *pT = last10;
    }
  }
}




int
main(int argc, char **argv) {
  int removed = 0;

  if (argc != 2) {
    fprintf(stderr, "usage: %s input.fasta\n", argv[0]);
    exit(1);
  }

  FastABuffer  B;
  FastA        I(argv[1]);

  const unsigned char  *SEQ;
  unsigned int          alloc = 1024 * 1024;
  unsigned char        *seq   = new unsigned char [alloc];
  unsigned char        *prn;
  unsigned int          len;
  int                   pA, pT;
  int                   i;

  for (I.first(B); !I.eof(); I.next(B)) {
    len = B.sequenceLength();
    SEQ = B.sequence();

    if (len >= alloc) {
      delete [] seq;
      alloc += len;
      seq = new unsigned char [alloc];
    }

    for (i=0; i<len; i++)
      seq[i] = SEQ[i];
    seq[i] = 0;

    pT = 0;
    pA = 0;

    get_polyAT(seq, len, &pT, &pA);

    if (pA > len) {
      fprintf(stderr, "pA = %d\nlen= %d\n", pA, len);
    }

    //fprintf(stdout, "BEFORE %d %d:\n%s\n", pT, pA, seq);

#if 0
#if 0
    for (i=0; i<pT; i++)
      seq[i] = 'N';
    for (i=len-1; i>len-1-pA; i--)
      seq[i] = 'N';

    prn = seq;
#else
    seq[len-pA] = 0;
    prn         = seq + pT;
#endif
#else
    prn = seq;
#endif

    fprintf(stdout, "%s\tpT=%d\tpA=%d\n", B.header(), pT, len-pA);
  }

  delete [] seq;

  fprintf(stderr, "Removed %d short ESTs.\n", removed);
}

