// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "fasta.H"

FastAWrapper::FastAWrapper(const char *filename, u32bit bufferSize) {
  _theSeqs               = 0L;
  _theNamesLen           = 0;
  _theNames              = 0L;
  _filebuffer            = 0L;
  _filename              = 0L;
  _indexname             = 0L;
  _currentSequenceNumber = 0;
  _isStreamInput         = false;
  _isRandomAccess        = false;

  if ((filename == 0L) || (strcmp(filename, "-") == 0)) {
    _filename = new char [6];
    strcpy(_filename, "stdin");

    if (bufferSize != ~u32bitZERO)
      _filebuffer    = new readBuffer(fileno(stdin), _filename, bufferSize);
    else
      _filebuffer    = new readBuffer(fileno(stdin), _filename);
    _isStreamInput = true;
  } else {
    _filename = new char [strlen(filename)+1];
    strcpy(_filename, filename);

#ifdef _AIX
    if (bufferSize != ~u32bitZERO)
      _filebuffer    = new readBuffer(_filename, bufferSize);
    else
      _filebuffer    = new readBuffer(_filename, 0);
#else
    if (bufferSize != ~u32bitZERO)
      _filebuffer    = new readBuffer(_filename, bufferSize);
    else
      _filebuffer    = new readBuffer(_filename);
#endif
    _isStreamInput = false;
  }
}

FastAWrapper::~FastAWrapper() {
  delete [] _filename;
  delete [] _indexname;
  delete    _filebuffer;
  delete [] _theSeqs;
  delete [] _theNames;
}


u32bit
FastAWrapper::getNumberOfSequences(void) {
  if (_isRandomAccess)
    return(_theGlobalDesc._numberOfSequences);
  else
    return(0);
}

const char *
FastAWrapper::getSourceName(void) {
  return(_filename);
}

u32bit
FastAWrapper::sequenceLength(IID_t iid) {
  if (_isRandomAccess)
    return(_theSeqs[iid]._seqLen);
  else
    return(0);
}

u32bit
FastAWrapper::headerLength(IID_t iid) {
  if (_isRandomAccess)
    return(_theSeqs[iid]._headerLen);
  else
    return(0);
}

u32bit
FastAWrapper::currentIID(void) {
  return(_currentSequenceNumber);
}
