// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "bri++.H"
#include "fasta-c.h"

typedef struct {
  FastAWrapper        *F;
  FastASequenceInCore *B;
} _fasta_c_private;

void*
createFastA(char *file) {
  _fasta_c_private  *c = new _fasta_c_private;
  c->F = new FastAWrapper(file);
  c->F->openIndex();
  c->B = 0L;
  return(c);
}

void
destroyFastA(void *c) {
  _fasta_c_private *C = (_fasta_c_private *)c;
  delete C->F;
  delete C->B;
  delete C;
}

char *
getFastAsequence(void *c, int idx) {
  _fasta_c_private *C = (_fasta_c_private *)c;

  if (C->B == 0L) {
    if (C->F->find(idx) == false) {
      fprintf(stderr, "Can't find iid=%u in %s\n", idx, C->F->getSourceName());
      exit(1);
    }
    C->B = C->F->getSequence();
  }

  if (C->B->getIID() != (u32bit)idx) {
    delete C->B;
    if (C->F->find(idx) == false) {
      fprintf(stderr, "Can't find iid=%u in %s\n", idx, C->F->getSourceName());
      exit(1);
    }
    C->B = C->F->getSequence();
  }

  return(C->B->sequence());
}

int
getFastAsequenceLength(void *c, int idx) {
  _fasta_c_private *C = (_fasta_c_private *)c;

  return(C->F->sequenceLength(idx));
}

char *
getFastAheader(void *c, int idx) {
  _fasta_c_private *C = (_fasta_c_private *)c;

  if (C->B == 0L) {
    if (C->F->find(idx) == false) {
      fprintf(stderr, "Can't find iid=%u in %s\n", idx, C->F->getSourceName());
      exit(1);
    }
    C->B = C->F->getSequence();
  }

  if (C->B->getIID() != (u32bit)idx) {
    delete C->B;
    if (C->F->find(idx) == false) {
      fprintf(stderr, "Can't find iid=%u in %s\n", idx, C->F->getSourceName());
      exit(1);
    }
    C->B = C->F->getSequence();
  }

  return(C->B->header());
}
