// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "fasta-cache.H"


FastACache::FastACache(char *filename, u32bit cachesize, bool loadall, bool report) {
  _fastawrapper = new FastAWrapper(filename);
  _fastawrapper->openIndex();

  if (loadall == false) {
    _allSequencesLoaded = false;
    _reportLoading      = report;
  
    _cacheMap  = new u32bit [_fastawrapper->getNumberOfSequences()];

    for (u32bit i=_fastawrapper->getNumberOfSequences(); i--; )
      _cacheMap[i] = ~u32bitZERO;

    _cacheSize = cachesize;
    _cacheNext = 0;
    _cache     = new FastASequenceInCore* [_cacheSize];

    for (u32bit i=0; i<_cacheSize; i++)
      _cache[i] = 0L;
  } else {
    _allSequencesLoaded = true;
    _reportLoading      = false;

    _cacheMap  = 0L;
    _cacheSize = _fastawrapper->getNumberOfSequences();
    _cacheNext = 0;
    _cache     = new FastASequenceInCore* [_cacheSize];

    fprintf(stderr, "Loading %u sequences from '%s'\n",
            _cacheSize,
            filename);

    for (u32bit i=0; i<_cacheSize; i++)
      _cache[i] = _fastawrapper->getSequence();
  }
}


FastACache::~FastACache() {
  for (u32bit i=0; i<_cacheSize; i++)
    delete _cache[i];
  delete [] _cache;
  delete [] _cacheMap;
  delete    _fastawrapper;
}



FastASequenceInCore*
FastACache::getSequence(u32bit iid)  {

  if (_allSequencesLoaded) {
    if (iid < _cacheSize) {
      return(_cache[iid]);
    } else {
      fprintf(stderr, "ERROR: FastACache of '%s' was asked for iid=%u, but only %u available.\n",
              _fastawrapper->getSourceName(), iid, _cacheSize);
      return(0L);
    }
  }

  //  Not all sequences loaded.

  if (_cacheMap[iid] != ~u32bitZERO)
    return(_cache[_cacheMap[iid]]);

  //  The sequence we are looking for isn't loaded.  If there
  //  isn't space in the cache, make space.

  if (_reportLoading)
    fprintf(stderr, "FastACache::getSequence()-- %s:"u32bitFMT" isn't loaded -- loading.\n",
            _fastawrapper->getSourceName(), iid);

  if (_cache[_cacheNext]) {
    _cacheMap[ _cache[_cacheNext]->getIID() ] = ~u32bitZERO;
    delete _cache[_cacheNext];
  }

  //  Load the sequence into the cache

  _fastawrapper->find(iid);
  FastASequenceInCore *ret = _fastawrapper->getSequence();

  _cache[_cacheNext] = ret;
  _cacheMap[iid] = _cacheNext;
  _cacheNext++;

  if (_cacheNext >= _cacheSize)
    _cacheNext = 0;

  //fprintf(stderr, "FastACache::getSequence()-- "u32bitFMT" loaded.\n", iid);

  //  Finally, return the sequence.

  return(ret);
}

